use std::fs::File;
use std::path::Path;
use std::io::prelude::*;
use regex::Regex;

use crate::common;
use crate::bar_modules;

#[derive(Clone)]
pub struct UnibarModuleMemory {
    opts:           common::AppOptions,
    meminfo_str:    String,
}

impl UnibarModuleMemory {

    // --------------------
    pub fn new(o :common::AppOptions) -> Self {
        UnibarModuleMemory {
            opts: o,
            meminfo_str: "".to_string(),
        }
    }
}

impl bar_modules::BarModuleActions for UnibarModuleMemory {

    // --------------------
    fn generate_data(&mut self) {
        let path = Path::new("/proc/meminfo");

        // Contents of '/proc/meminfo' has memory information
        let mut meminfo_file = match File::open(&path) {
            Err(why) => panic!("couldn't open {}: {}", path.display(), why),
            Ok(file) => file,
        };

        match meminfo_file.read_to_string(&mut self.meminfo_str) {
            Err(why) => panic!("couldn't read {}: {}", path.display(), why),
            Ok(_) => if self.opts.debug {
                println!("-----> meminfo - {:#?}", self.meminfo_str);
            },
        };
        self.meminfo_str.pop();

    }

    // --------------------
    // MemTotal:        7822812 kB\nMemFree:          399244 kB\nMemAvailable:    3986504 kB
    fn get_content(&self) -> String {
        let re = Regex::new(r"MemTotal:\s+(\d+)\s+kB\nMemFree:\s+(\d+)\s+kB\nMemAvailable:\s+(\d+)").unwrap();
        let caps = re.captures(self.meminfo_str.as_str()).unwrap();

        let total_mem :f32 = caps.get(1).unwrap().as_str().parse::<f32>().unwrap();
        let avail_mem :f32 = caps.get(3).unwrap().as_str().parse::<f32>().unwrap();
        return format!("{}%", ((total_mem - avail_mem)/total_mem * 100.0).ceil() as i32);
    }

    // --------------------
    fn get_icon(&self) -> String {
        return "💾".to_string();
    }
}

impl bar_modules::BarModuleDebug for UnibarModuleMemory {

    // --------------------
    fn post_debug(&self) {
    }
}
